function [hprof c A1 A2] = EITCSim(hbar,mu,e0,e1,gamma,tau,tbar,st,gens)
%EITCSim: Run EITC simulation in which tax changes at time tbar
%To do so, computes hours profile and consumption going forward from tbar
%with assets st at tbar, e0,e1,hbar,mu,gamma given, and taxes tau from tbar on
%with gens generations
    
    %initialize emax
    emax = e0;
    if(tbar>.5)
        emax = e0-2*(e0-e1)*(tbar-.5);
    end
    
    %parts of the integral
    part1=@(l) (etwid(l,tbar,e0,e1)).^(1+1/gamma);
    part2=@(l) etwid(l,tbar,e0,e1);
    %golden section algorithm
    h0min=hbar;h0max=1; %hmax is on [hbar,1]
    while (h0max-h0min)> .0000001
        h0n=.5*(h0min+h0max); %hmax midpoint
        % caluclate lamda_star at midpoint
        lamn=etwidinv(emax*((hbar/h0n)*((1+gamma)/gamma))^gamma,tbar,e0,e1);
        % calculate left hand side and right hand side of equation used to calibrate mu using lamn.
        % If rhs>lhs bring down h0max, if lhs<rhs bring up h0min
        lhsn=(1-tau)*(1-tbar)*emax./h0n.^gamma/mu;
        rhsn=st+h0n/(emax^(1/gamma))*simint(part1,0,lamn,.00001)-hbar*simint(part2,0,lamn,.00001);
        if rhsn > lhsn, h0max=h0n;else h0min=h0n;end
    end
    lam=lamn;h0=h0n; %define true lamda and hnot
    
    %censor lamda and fix hours profile
    if lam<0, lam=0; end %hours profile is irrelevant and does not need to be fixed
    if lam>1-tbar
        lam=1-tbar;
        %only second FOC holds. Solve it for h0
        h0min=hbar;h0max=1; %hmax is on [hbar,1]
        while (h0max-h0min)> .0000001
            h0n=.5*(h0min+h0max); %hmax midpoint
            % calculate left hand side and right hand side of equation used to calibrate mu using lamn.
            % If rhs>lhs bring down h0max, if lhs<rhs bring up h0min
            lhsn=(1-tau)*(1-tbar)*emax./h0n.^gamma/mu;
            rhsn=st+h0n/(emax^(1/gamma))*simint(part1,0,lam,.00001)-hbar*simint(part2,0,lam,.00001);
            if rhsn > lhsn, h0max=h0n;else h0min=h0n;end
        end
        h0=h0n;
    end
    
    %compute consumption
    y=st+h0/(emax^(1/gamma))*simint(part1,0,lam,.00001)-hbar*simint(part2,0,lam,.00001); %income for rest of life net of assets
    c=y/(1-tbar);
    
    %compute entry and retirement date    
    if lam<1-2*tbar
        A1=.5-lam/2;
        A2=.5+lam/2;
    else
        A1=tbar;
        A2=lam+tbar;
    end
    
    %compute hours profile
    step=1/(gens);
    time=0:step:1;
    hprof = h0*(etwid(LamofA(time,tbar),tbar,e0,e1)./emax).^(1/gamma);
    hprof(time<=A1)=0;
    hprof(time>=A2)=0;
end

