function [hprof c E0 R0 E1 R1 error] = IcelandSim(hbar,mu,e0,e1,gamma,tau0,tau1,t0,t1,st,gens)
%IcelandSim: Run Iceland simulation
%Computes solution going forward from t0 with tax changing to tau0 at t0
%and then to tau1 at t1, with initial assets st and gens generations.
%Returns hours profile, consumption, and entry and retirement dates for
%the periods between t0 and t1 and after t1.

step=1/gens;
error=0; %error flag
corner=0; %corner solution for hours above 1 flag
    
%if t1>=1 the problem reduces to the EITC sim
if(t1>=1)
    [hprof c E0 R0]=EITCSim(hbar,mu,e0,e1,gamma,tau0,t0,st,gens);
    E1=0; R1=0;
else %otherwise we need so solve everything
    %initialize emax0 and emax1
    emax1 = e0;
    if(t1>.5)
        emax1 = e0-2*(e0-e1)*(t1-.5);
    end
    
    emax0 = e0;
    if(t1<=.5)
        emax0=e1+2*(e0-e1)*t1;
    elseif(t0>=.5)
        emax0=2*e0-e1-2*t0*(e0-e1);
    end
    
    %etwid is etwid0 in part 0
    part1=@(l) (etwid0(l,t0,t1,e0,e1)).^(1+1/gamma);
    part2=@(l) etwid0(l,t0,t1,e0,e1);
    %etwid is standard etwid from EITC in part 1
    part3=@(l) (etwid(l,t1,e0,e1)).^(1+1/gamma);
    part4=@(l) etwid(l,t1,e0,e1);
    
    %solve unconstrained system in which individual does not work all of
    %time between t0 and t1. Fsolve is fickle, so if one initial value does
    %not work try another. If exhaust all return error flag.
    systemunconstrained=@(i) IcelandSystemUnconstrained(i(1),i(2),i(3),i(4),hbar,mu,e0,e1,gamma,tau0,tau1,t0,t1,st,emax0,emax1);      
    [x val exit]=fsolve(systemunconstrained, [.3, .3, 0, .3]);
    if(exit<1 || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0 || imag(x(4))~=0) %solver did not work or returned immaginary number
        [x val exit]=fsolve(systemunconstrained, [.3, .3, .3, .3]); %try different initial value
        if(exit<1 || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0 || imag(x(4))~=0) %solver did not work or returned immaginary number
            [x val exit]=fsolve(systemunconstrained, [.3, .3, .1, .3]); %try different initial value
            if(exit<1 || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0 || imag(x(4))~=0) %solver did not work or returned immaginary number
                [x val exit]=fsolve(systemunconstrained, [.3, .3, .2, .2]) %try different initial value
                if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0|| imag(x(4))~=0) %solver did not work or returned immaginary number
                    [x val exit]=fsolve(systemunconstrained, [.2, .2, 0, .1]) %try different initial value
                    if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0|| imag(x(4))~=0) %solver did not work or returned immaginary number
                        [x val exit]=fsolve(systemunconstrained, [.3, .3, .0000001, .24]) %try different initial value
                        if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0|| imag(x(4))~=0) %solver did not work or returned immaginary number
                            [x val exit]=fsolve(systemunconstrained, [.3, .3, 0, 0]) %try different initial value
                            if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0|| imag(x(4))~=0) %solver did not work or returned immaginary number
                                [x val exit]=fsolve(systemunconstrained, [.3, .3, 0, 0]) %try different initial value
                                if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0|| imag(x(4))~=0) %solver did not work or returned immaginary number
                                    [x val exit]=fsolve(systemunconstrained, [.4, .4, .0001, 0]) %try different initial value
                                    if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0|| imag(x(4))~=0) %solver did not work or returned immaginary number
                                        [x val exit]=fsolve(systemunconstrained, [.5, .5, .0001, 0]) %try different initial value
                                        if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0|| imag(x(4))~=0) %solver did not work or returned immaginary number
                                            [x val exit]=fsolve(systemunconstrained, [.45, .45, 0,.000001])
                                            if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0|| imag(x(4))~=0) %solver did not work or returned immaginary number
                                                [x val exit]=fsolve(systemunconstrained, [.25, .25, .00000001,.0000000001])
                                                if((exit<0  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0)|| imag(x(4))~=0) %solver did not work or returned immaginary number
                                                    [x val exit]=fsolve(systemunconstrained, [.35, .35, .00000001,.0000000001])
                                                    if(exit<0  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0|| imag(x(4))~=0) %solver did not work or returned immaginary number
                                                        [x val exit]=fsolve(systemunconstrained, [.3, .3, .00000001,.0000000001])
                                                        if(exit<0  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0|| imag(x(4))~=0) %solver did not work or returned immaginary number
                                                            [x val exit]=fsolve(systemunconstrained, [.4, .4, .00000001,.0000000001])
                                                            if(exit<0  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0|| imag(x(4))~=0) %solver did not work or returned immaginary number
                                                                [x val exit]=fsolve(systemunconstrained, [.45, .45, 0,.235])
                                                                if(exit<0  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0|| imag(x(4))~=0) %solver did not work or returned immaginary number
                                                                    error=1 % got through everything and still did not work -- return error flag
                                                                    t0 %for debug, print time of failure
                                                                end
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
    %extract solution from x
    hmax0=x(1);
    hmax1=x(2);
    lamda0=x(3);
    lamda1=x(4);
    %censor lamda
    if(lamda0<0) lamda0=0; end;
    if(lamda1<0) lamda1=0; end;
    if(lamda0<=t1-t0 && hmax0>1)
        error=1 %corner solution that can't be solved analytically
    end

    %if lamda0 is too big, solve constrained system with lamda0=t1-t0
    if(lamda0>t1-t0)
        systemconstrained=@(i) IcelandSystemConstrained(i(1),i(2),i(3),hbar,mu,e0,e1,gamma,tau0,tau1,t0,t1,st,emax0,emax1);
        [x val exit]=fsolve(systemconstrained,[.3,.3,0]);
        if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0) %solver did not work or returned immaginary number
            [x val exit]=fsolve(systemconstrained,[.3,.3,.3]);%try different initial value
            if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0) %solver did not work or returned immaginary number
                   [x val exit]=fsolve(systemconstrained,[.3,.3,.1]);%try different initial value
                if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0) %solver did not work or returned immaginary number
                        [x val exit]=fsolve(systemconstrained,[.3,.3,.01]); %try different initial value
                        if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0) %solver did not work or returned immaginary number
                            [x val exit]=fsolve(systemconstrained,[.5,.5,.4]); %try different initial value
                            if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0) %solver did not work or returned immaginary number
                                [x val exit]=fsolve(systemconstrained,[.5,.5,.3]); %try different initial value
                                if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0) %solver did not work or returned immaginary number
                                    [x val exit]=fsolve(systemconstrained,[.5,.5,.2]); %try different initial value
                                    if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0) %solver did not work or returned immaginary number
                                        error=1 % got through everything and still did not work -- return error flag
                                        t0 %for debug, print time of failure
                                    end
                                end
                            end
                        end
                end
            end
        end
    
        lamda0=t1-t0;
        hmax0=x(1);
        hmax1=x(2);
        lamda1=x(3);
        if(lamda1<0) lamda1=0; end;
        
        %if lamda1 is too big, solve constrained system with both lamda0
        %and lamda1 constrained
        if(lamda1>1-t1)
            %solve constrained system with both lamda0 and lamda1 both constrained
            systemconstrained3=@(i) IcelandSystemConstrained3(i(1),i(2),hbar,mu,e0,e1,gamma,tau0,tau1,t0,t1,st,emax0,emax1);
            [x val exit]=fsolve(systemconstrained3, [.3,.3]);
            
            if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0) %solver did not work
                error=1 % return error flag
            end
            
            lamda0=t1-t0;
            lamda1=1-t1;
            hmax0=x(1);
            hmax1=x(2);
        elseif(hmax0>1) %corner solution for hours
           corner=1;
           emin0=etwid0(t1-t0,t0,t1,e0,e1);
           systemcorner1=@(i) IcelandSystemCorner1(i(1),i(2),i(3),hbar,mu,e0,e1,gamma,tau0,tau1,t0,t1,st,emin0,emax1);
           systemcorner2=@(i) IcelandSystemCorner2(i(1),i(2),hbar,mu,e0,e1,gamma,tau0,tau1,t0,t1,st,emax1);
           z=3;
           
           [x val exit]=fsolve(systemcorner1,  [.9,.4,.2]);
           if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0) %solver did not work or returned immaginary number
               [x val exit]=fsolve(systemcorner1,  [.9,.4,.1]);
               if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0) %solver did not work or returned immaginary number
                    [x val exit]=fsolve(systemcorner1,  [.9,.4,.00001]);
                    if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0) %solver did not work or returned immaginary number
                        error=1
                    end
               end
           end
           
           lamda0=t1-t0;
           hmin0=x(1);
           hmax1=x(2);
           lamda1=x(3);
           if(hmin0>1)
               [x val exit]=fsolve(systemcorner2,  [.4,.1]);
               if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0) %solver did not work or returned immaginary number
                   error=1
               end
               lamda0=t1-t0;
               hmin0=1;
               hmax1=x(1);
               lamda1=x(2);
           end
        end
    %if lamda1 is too big, solve constrained system with lamda1=1-t1
    elseif(lamda1>1-t1)
        
        systemconstrained2=@(i) IcelandSystemConstrained2(i(1),i(2),i(3),hbar,mu,e0,e1,gamma,tau0,tau1,t0,t1,st,emax0,emax1);
        x=fsolve(systemconstrained2, [.3,.3,.3]);
        if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0) %solver did not work or returned immaginary number
            [x val exit]=fsolve(systemconstrained2,[.3,.3,0]);%try different initial value
            if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0) %solver did not work or returned immaginary number
                   [x val exit]=fsolve(systemconstrained2,[.3,.3,.1]);%try different initial value
                if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0 || imag(x(3))~=0) %solver did not work or returned immaginary number
                    error=1 % got through everything and still did not work -- return error flag
                end
            end
        end
        
        lamda1=1-t1;
        hmax0=x(1);
        hmax1=x(2);
        lamda0=(3);
        if(lamda0<0) lamda0=0; end;
        if(lamda0<=t1-t0 && hmax0>1)
            error=1 %corner solution that can't be solved analytically
        end
        %if lamda0 is too big, solve constrained system with both lamda0
        %and lamda1 both constrained
        if(lamda0>t1-t0)
            systemconstrained3=@(i) IcelandSystemConstrained3(i(1),i(2),hbar,mu,e0,e1,gamma,tau0,tau1,t0,t1,st,emax0,emax1);
            [x val exit]=fsolve(systemconstrained3, [.3,.3]);
            if(exit<1  || imag(x(1))~=0 || imag(x(2))~=0) %solver did not work  or returned immaginary number
                error=1 %return error flag
            end
                        
            lamda0=t1-t0;
            lamda1=1-t1;
            hmax0=x(1);
            hmax1=x(2);
        end
    end
    
    %compute consumption
    y=hmax0/(emax0^(1/gamma))*simint(part1,0,lamda0,.0001)-hbar*simint(part2,0,lamda0,.0001)...
                +hmax1/(emax1^(1/gamma))*simint(part3,0,lamda1,.0001)-hbar*simint(part4,0,lamda1,.0001)+st;
    c=y/(1-t0);
    
    %compute entry and retirement dates
    if(lamda0==0)
        E0=1; R0=1;
    else
        if(t1<=.5)
            E0=t1-lamda0;
            R0=t1;
        elseif(t0>=.5)
            E0=t0;
            R0=t0+lamda0;
        else
            if(.5-t0<t1-.5)
                if(lamda0<2*(.5-t0))
                    E0=.5-lamda0/2;
                    R0=.5+lamda0/2;
                else
                    E0=t0;
                    R0=t0+lamda0;
                end
            else
                if(lamda0<2*(t1-.5))
                    E0=.5-lamda0/2;
                    R0=.5+lamda0/2;
                else
                    R0=t1;
                    E0=t1-lamda0;
                end
            end
        end
    end
    
    if(lamda1==0)
        E1=1; R1=1;
    else
        if lamda1<1-2*t1
            E1=.5-lamda1/2;
            R1=.5+lamda1/2;
        else
            E1=t1;
            R1=lamda1+t1;
        end
    end
    
    %compute hours profile
    time=0:step:1;
    
    %hours profile between t0 and t1
    if(corner==0) %interior solution for hours
        lamofa0=LamofA0(time,t0,t1);
        hprof0=hmax0*(etwid0(lamofa0,t0,t1,e0,e1)./emax0).^(1/gamma);
        hprof0(find(time<=E0))=0;
        hprof0(find(time>R0))=0;
    else
        lamofa0=LamofA0(time,t0,t1);
        hprof0=min(hmin0*(etwid0(lamofa0,t0,t1,e0,e1)./emin0).^(1/gamma),ones(size(lamofa0)));
        hprof0(find(time<=E0))=0;
        hprof0(find(time>R0))=0;
    end
    
    %hours profile after t1
    lamofa1 = LamofA(time,t1); %lamda(a)
    hprof1 = hmax1*(etwid(lamofa1,t1,e0,e1)./emax1).^(1/gamma);
    hprof1(find(time<=E1))=0;
    hprof1(find(time>R1))=0;
    
    %combine them
    hprof=hprof0+hprof1;
end
end

