function [hoursprof lfpprof nhoursprof nlfpprof weeklylfp maxhours minhours] = IcelandSimOLG(hbar,mu,e0,e1,gamma,tau,taun,taun2,taulength,daysperweek)
%IcelandSimOLG: Do Iceland Sim for daysperweek * 52 * 60 generations
%and add across generations. Tax rate is initially tau. At time zero,
%changes to taun for taulength. Then reverts to taun2.

%To do this, for given hbar, e0, e1, gamma, and mu computes asset profile
%and then solves IcelandSim forward for each generation. Then adds across generations
%to get path of LFP and overall hours profile over time with and without
%the tax change.

%There are weeks*daysperweek generations, with each time period
%representing a day. Data are binned to count whether an individual worked
%each week and then aggregated up to fraction of weeks worked per year.

gens=52*60*daysperweek;
step=1/gens;
gens=gens+1; %add one because inclusive of last period
time=0:step:1;

%First simulate the model for the given hbar, e0, e1, gamma, and mu to get
%the asset profile
[ohprof s]=RWSim(hbar,mu,e0,e1,gamma,tau,gens-1);
olprof =(ohprof>0);

%Then simulate the model with tau2n to get the hours profile for
%generations that enter after the reform (e for entering generations)
ehprof=RWSim(hbar,mu,e0,e1,gamma,taun2,gens-1);
elprof=(ehprof>0);

hoursprof=zeros(1,gens);
lfpprof=zeros(1,gens);
%compute hoursprof and lfpprof with no tax change for overlapping generations
for(i=1:gens)
    hoursprof=hoursprof+1/gens*[ohprof(i:gens) ohprof(1:i-1)];
    lfpprof=lfpprof+1/gens*[olprof(i:gens) olprof(1:i-1)];
end

%Loop through each generation and compute hours and LFP with tax = taun for
%taulength and then return to taun2 afterwards
nhoursprof=zeros(1,gens); %aggregate hours profile across generations
nlfpprof=zeros(1,gens); %aggregate lfp profile across generations
weeklylfp=zeros(120*52,2); %weekly lfp across generations
weeklylfp(:,1)=[-60:(1/52):(3119/52)]'; %first column is dates
weekly=zeros(120*52,1);
maxhours=0; minhours=1;
for(i=1:gens)
    t0=time(i); % age at which generation reoptimizes (initial tax change)
    t1=t0+taulength; %age at which tax reverts
    %run the simulation for outgoing generation age t when tax change occurs
    [nhprof nc E0 R0 E1 R1 error]=IcelandSim(hbar,mu,e0,e1,gamma,taun,taun2,t0,t1,s(i),gens-1);
    if(error==1)
        error=1
        break;
    end
    nlprof=(nhprof>0); %lfpprof
    maxhours=max(maxhours,max(nhprof));
    if(max(nhprof)>0)
        minhours=min(minhours,min(nhprof(nhprof>0)));
    end
    if(1-t0<taulength) %if you don't see the tax change back during your life, the incoming generation has the low tax for some amount of time
        [nhprof2 nc E0 R0 E1 R1 error]=IcelandSim(hbar,mu,e0,e1,gamma,taun,taun2,0,taulength-(1-t0),s(i),gens-1);
        if(error==1)
            error=1
            break;
        end
        nlprof2=(nhprof2>0);
        maxhours=max(maxhours,max(nhprof2));
        if(max(nhprof2)>0)
            minhours=min(minhours,min(nhprof2(nhprof2>0)));
        end
        
        %add generation and generatino that replaces it on to aggregae
        %hours profile (weight generation 1/1001)
        nhoursprof=nhoursprof+(1/gens*[nhprof(i:gens) nhprof2(1:i-1)]);
        nlfpprof=nlfpprof+(1/gens*[nlprof(i:gens) nlprof2(1:i-1)]);
        
        %build weekly bins
        fulllfpprof=[olprof(i:gens) olprof(1:i-1) nlprof(i:gens) nlprof2(1:i-1)];
        for(j=0:(52*120-1))
            weekly(j+1)=max(fulllfpprof((j*daysperweek+1):((j+1)*daysperweek)));
        end
        weeklylfp(:,2)=weeklylfp(:,2)+1/gens*weekly;
    else
       %add generation and generation that replaces it on to aggregate
       %hours profile (weight generation 1/gens)
       nhoursprof=nhoursprof+(1/gens*[nhprof(i:gens) ehprof(1:i-1)]);
       nlfpprof=nlfpprof+(1/gens*[nlprof(i:gens) elprof(1:i-1)]);
       
       %build weekly bins
        fulllfpprof=[olprof(i:gens) olprof(1:i-1) nlprof(i:gens) elprof(1:i-1)];
        for(j=0:(52*120-1))
            weekly(j+1)=max(fulllfpprof((j*daysperweek+1):((j+1)*daysperweek)));
        end
        weeklylfp(:,2)=weeklylfp(:,2)+1/gens*weekly;
    end
end
end

