function [hoursprof assetprof] = IcelandSim_iterative_debug(hbar,mu,e0,e1,gamma,tau0,tau1,T0,T1,gens,st,svec,hvec,stp1mat,hmat,ghmat0, ghmat1,diffmat,evec,vht,u)
%Iceland Sim_iterative: Simulate overlapping generations of Icleand temp tax change
%hbar, mu, e0, e1, gamma, tau0, tau1, st given. Assets are st at change at
%period number T0. At T1 it changes to tau1.

%set up
step=1/(gens-1);
time=0:step:1;
ss=size(svec); ss=ss(1);
sh=size(hvec); sh=sh(1);
%matrices passed in

%Get transfer policy from EITCSim
icelandhours=IcelandSim(hbar,mu,e0,e1,gamma,tau0, tau1,(T0-2)*step,(T1-2)*step,st,gens-1);
tauvec=zeros(1,gens);
if(T0>0)
    tauvec(T0:T1-1)=tau0;
end
tauvec(T1:gens)=tau1;
trans=step*evec'*max(icelandhours-hbar,0);
trans=diag(trans);
trans=trans*tauvec;
trans=diag(trans);


%Terminal period: just choose hours
if(T1<=gens)
    vmat=step*u(diffmat+evec(gens)*ghmat1+repmat(trans(gens),ss*sh,ss))-vht; %value function for each choice of assets and hours
else
    vmat=step*u(diffmat+evec(gens)*ghmat0+repmat(trans(gens),ss*sh,ss))-vht; %value function for each choice of assets and hours
end
vmat(stp1mat~=0)=-inf; %force to choose zero assets in terminal period. Requies step to be such that zero assets is an option
[vals indices]=max(vmat); %maximize across colums to get value functions and indices of optima
hoursmat(:,gens)=hmat(indices,1);
valmat(:,gens)=vals';

%backwards Bellman iteration
for(t=gens-1:-1:max(T0,1))
    if(t>=T1)
        vmat=step*u(diffmat+evec(t)*ghmat1+repmat(trans(t),ss*sh,ss))-vht+repmat(valmat(:,t+1),sh,ss); %value function for each choice of assets and hours
    else
        vmat=step*u(diffmat+evec(t)*ghmat0+repmat(trans(t),ss*sh,ss))-vht+repmat(valmat(:,t+1),sh,ss); %value function for each choice of assets and hours
    end
    [vals indices]=max(vmat); %maximize across colums to get value functions and indices of optima
    hoursmat(:,t)=hmat(indices,1);
    assetsmat(:,t)=stp1mat(indices,1);
    valmat(:,t)=vals;
    choicemat(:,t)=mod(indices-1,ss)+1;
end

%trace through assetmat to get optimal choices, then use this to build
%hours and asset profiles
assetindexprof=zeros(gens,1);
assetprof=zeros(gens,1);
hoursprof=zeros(gens,1);
assetindexprof(max(T0,1))=find(svec==st);
hoursprof(max(T0,1))=hoursmat(assetindexprof(max(T0,1)),max(T0,1));
for(t=max(T0,1)+1:gens)
    assetindexprof(t)=choicemat(assetindexprof(t-1),t-1);
    assetprof(t)=svec(assetindexprof(t));
    hoursprof(t)=hoursmat(assetindexprof(t),t);
end
end
