%Iceland.m: Simulates Iceland and produces simulation data with hbar
%heterogeneity for comparison with Bianchi Et Al (2001)

daysperweek=3;
gens=60*52*daysperweek;
step=1/gens;
gens=gens+1;

time=0:step:1; %
gamma=4; %preference parameter in the utility function
e0=1;e1=.599; %intercept and slope of age-productivity profile.

%calibration of mu and hbar - straight from Rogerson-Wallenius code
h0t=.7; %target value for hmmax, so that 45% of discretionary time per week is worked
lamt=.792;%target value for share of lifetime working
taut=.145; %initial average tax rate = 14.5%  (Bianchi)
taun=0; %0 tax in 1987
taun2=.08; % 8% tax after
hbar=((e0*(1-lamt)+e1*lamt)/e0)^(1/gamma)*h0t*gamma/(1+gamma); %formula to
%calibrate hbar
mua=e0/h0t^gamma/(e0*h0t/(1/gamma+2).*(1-(1-lamt).^(1/gamma+2))-e0*hbar/2*(1-(1-lamt).^2)); %not sure what this is
mu=(1-taut)*e0/h0t^gamma/...
    (h0t/e0^(1/gamma)/(1/gamma+2)/(e1-e0).*((e0*(1-lamt)+e1*lamt).^(1/gamma+2)-e0^(1/gamma+2))-...
    hbar/2/(e1-e0).*((e0*(1-lamt)+e1*lamt).^2-e0^2)); %Really ugly expression for alpha (they call mu here, but alpha in paper)

% wage ratio
vec=0:.0001:lamt;
eproft=e0*(1-vec)+e1*vec;
hproft=h0t*((1-vec)+e1/e0*vec).^(1/gamma);
wproft=eproft.*(hproft-hbar)./hproft;
s=size(vec);
s=s(2);
wratio=wproft(s)/wproft(1)

% simulate elasticities
el_gens=6000; %gens for computing elasticities
[intensive_frisch participation_frisch aggregate_frisch] = SimFrischElasticity(hbar,mu,e0,e1,gamma,taut,taut-.01,el_gens);
h1=RWSim(hbar,mu,e0,e1,gamma,taut,el_gens);
h2=RWSim(hbar,mu,e0,e1,gamma,taut-.01,el_gens);
h1=1/el_gens*sum(h1);
h2=1/el_gens*sum(h2);
aggregate_hicksian=log(h2/h1)/log((1-taut+.01)/(1-taut));
h1=RWSim(hbar,mu,e0,e1,gamma,taut,el_gens);
h2=RWSim(hbar,mu,e0,e1,gamma,taut-.01,el_gens);
h1=(h1>0);
h2=(h2>0);
h1=1/el_gens*sum(h1);
h2=1/el_gens*sum(h2);
participation_hicksian=log(h2/h1)/log((1-taut+.01)/(1-taut));
h1=RWSim(hbar,mu,e0,e1,gamma,taut,el_gens);
h2=RWSim(hbar,mu,e0,e1,gamma,taut-.01,el_gens);
h2(h1==0)=0;
h1=1/el_gens*sum(h1);
h2=1/el_gens*sum(h2);
intensive_hicksian=log(h2/h1)/log((1-taut+.01)/(1-taut));

save('Iceland-point25.mat');

%run simulation
[hoursprof lfpprof nhoursprof nlfpprof weeklylfp maxhours minhours]=IcelandSimOLG(hbar,mu,e0,e1,gamma,taut,taun,taun2,1/60,daysperweek);

time2=[-60:(step*60):60];
nhoursprof2=[hoursprof nhoursprof(2:gens)];
hoursprof2=[hoursprof hoursprof(2:gens)];
nlfpprof2=[lfpprof nlfpprof(2:gens)];
lfpprof2=[lfpprof lfpprof(2:gens)];

save('Iceland-point25.mat');
%output in Iceland-unbinned.xlsx and Iceland-binned.xlsx
outmat=[time2' lfpprof2' nlfpprof2'];
csvwrite('Iceland-point25-unbinned.csv', outmat);
csvwrite('Iceland-point25-binned.csv',weeklylfp);

intensive_frisch
participation_frisch
aggregate_frisch
intensive_hicksian
participation_hicksian
aggregate_hicksian
hbar
mu
gamma
e1
maxhours
minhours
